@@ -15,6 +15,7 @@ IOSystems created:
     makeSinusoid: creates an IOSystem that outputs a sinusoid
 """
 
+from controlSBML import constants as cn
 from controlSBML import logger as lg
 from controlSBML import msgs
 
@@ -32,7 +33,7 @@ STATE = "state"
 
 class IOSystemFactory(object):
 
-    def __init__(self, name="factory", callback_fcn=None):
+    def __init__(self, name="factory", callback_fcn=None, dt=1/cn.POINTS_PER_TIME):
         """
         Parameters
         ----------
@@ -54,6 +55,7 @@ class IOSystemFactory(object):
         """
         self.name = name
         self.callback_fcn = callback_fcn
+        self.dt = dt
         self.registered_names = []  # List of names logged
         self.loggers = []  # Loggers for created objects
         self.callback_log = []
@@ -113,82 +115,6 @@ class IOSystemFactory(object):
         merged_logger = logger.merge(self.name, others)
         return merged_logger.report()
 
-    def oldmakePIDController(self, name, kp=2, ki=0, kd=0):
-        """
-        Creates a PID controller.
-        NonlinearIOSystem with input IN and output OUT.
-        States are:
-            culmulative_err: cumulative control error
-            last_err: last control error
-        
-        Parameters
-        ----------
-        name: str
-            Name of the system
-        kp: float
-           proportional control constant
-        ki: float
-           integral control constant
-        kd: float
-           differential control constant.
-        
-        Returns
-        -------
-        control.NonlinearIOSystem
-        """
-        X_CUMU = 0  # cumulative control error
-        X_LAST = 1 # last control error
-        X_TIME = 2
-        last_time = 0
-        logger = self._registerLogger(name, [IN, "last_err",
-              "acc_err", OUT])
-        dct = {"kp": kp, "ki": ki, "kd": kd}
-        global is_print
-        is_print = False
-        def updfcn(time, x_vec, u_vec, _):
-            global is_print
-            # Calculate the derivative of the state variables
-            u_val = self._array2scalar(u_vec)
-            last_time = x_vec[X_TIME]
-            last_err = x_vec[X_LAST]
-            last_cumu = x_vec[X_CUMU]
-            time = max(time, 1e-8)
-            # Calculate derivatives
-            dtime = time - last_time
-            dcumu = u_val/dtime
-            derr = (u_val - last_err)/dtime
-            if self.callback_fcn is not None:
-                call_name = "%s.updfcn" % name
-                self.callback_log.append(
-                      self.callback_fcn(call_name, time, x_vec, u_vec, dct,
-                      [dcumu, derr, dtime]))
-            if is_print:
-                print(time, x_vec, u_vec)
-            if time > 1:
-                is_print = True
-            return dcumu, derr, dtime
-        #
-        def outfcn(time, x_vec, u_vec, __):
-            # u: float (error signal)
-            u_val = self._array2scalar(u_vec)
-            cumu_err = x_vec[X_CUMU]
-            last_err = x_vec[X_LAST]
-            control_out =  kp*u_val  \
-                          + ki*cumu_err \
-                          + kd*(u_val - last_err)
-            logger.add(time, [u_val, x_vec[1], x_vec[0], control_out])
-            if self.callback_fcn is not None:
-                call_name = "%s.outfcn" % name
-                self.callback_log.append(
-                      self.callback_fcn(call_name, time, x_vec, u_vec, dct,
-                      [control_out]))
-            return control_out
-        #
-        return control.NonlinearIOSystem(
-            updfcn, outfcn, inputs=[IN], outputs=[OUT],
-            states=["cumulative_error", "last_error", "last_time"],
-            name=name)
-
     def makePIDController(self, name, kp=2, ki=0, kd=0):
         """
         Creates a PID controller.
@@ -212,30 +138,33 @@ class IOSystemFactory(object):
         -------
         control.NonlinearIOSystem
         """
-        global last_time, last_u_val, accumulated_u_val
-        last_time = 0
-        last_u_val = 0
-        accumulated_u_val = 0
         #
         logger = self._registerLogger(name, [IN, "last_err",
               "acc_err", OUT])
-        def outfcn(time, _, u_vec, __):
+        # FIXME: A non-zero ki causes an algebraic loop error
+        def updfcn(time, x_vec, u_vec, __):
             # u: float (error signal)
-            global last_time, last_u_val, accumulated_u_val
+            # x_vec[0] - last_u
+            # x_vec[1] - accumulated_u
             u_val = self._array2scalar(u_vec)
+            daccumulated_u = u_val/self.dt
+            dlast_u_val = (u_val - x_vec[0])/self.dt
+            return dlast_u_val, daccumulated_u
+        #
+        def outfcn(time, x_vec, u_vec, __):
+            # u: float (error signal)
+            u_val = self._array2scalar(u_vec)
+            last_u_val = x_vec[0]
+            accumulated_u_val = x_vec[1]
             control_out =  kp*u_val  \
                           + ki*accumulated_u_val \
-                          + kd*(u_val - last_u_val)
-            x_vec = [last_u_val, accumulated_u_val]
-            last_time = time
-            last_u_val = u_val
-            accumulated_u_val += u_val
+                          + kd*(u_val - last_u_val)/self.dt
             logger.add(time, [u_val, x_vec[0], x_vec[1], control_out])
-            #print(time, u_vec, control_out)
             return control_out
         #
         return control.NonlinearIOSystem(
-            None, outfcn, inputs=[IN], outputs=[OUT],
+            updfcn, outfcn, inputs=[IN], outputs=[OUT],
+            states=["last_err", "accumulated_err"],
             name=name)
 
     def makeFullStateController(self, name, ctlsb, factor=1.0, poles=-2, time=0):
@@ -253,7 +182,7 @@ class IOSystemFactory(object):
             factor for adjusting the reference input to get a closed loop
             transfer function of 1
         poles: list-float/float
-            Desired poles
+            Desired poles; single float is the dominant pole
         time: float
             Time where system is lienarized
         
@@ -268,10 +197,13 @@ class IOSystemFactory(object):
         """
         # Validity Checks
         if len(ctlsb.input_names) != 1:
-            raise ValueError("SBML model must have a single input. Has: %s" % str(ctlsb.input_names))
+            msg = "SBML model must have a single input. Has: %s"  \
+                  % str(ctlsb.input_names)
+            raise ValueError(msg)
         # Initializations
         state_space = ctlsb.makeStateSpace(time=time)
-        controller_input_names = [n for n in ctlsb.state_names if not n in ctlsb.input_names]
+        controller_input_names = [n for n in ctlsb.state_names
+             if not n in ctlsb.input_names]
         controller_input_names.insert(0, REF)  # first input
         num_state_input = len(controller_input_names) - 1
         is_distinct_poles = True
@@ -464,13 +396,15 @@ class IOSystemFactory(object):
         return control.NonlinearIOSystem(
             None, outfcn, outputs=[OUT], inputs=[], name=name)
 
-    def makePassthru(self, name):
+    def makePassthru(self, name, input_name=None, output_name=None):
         """
         Makes a pass through system that outputs its input.
         
         Parameters
         ----------
         name: str
+        input_name: str
+        output_name: str
         
         Returns
         -------
@@ -478,6 +412,10 @@ class IOSystemFactory(object):
         """
         logger = self._registerLogger(name, [IN, OUT])
         dct = {}
+        if input_name is None:
+            input_name = IN
+        if output_name is None:
+            output_name = OUT
         def outfcn(time, x_vec, u_vec, ___):
             u_val = self._array2scalar(u_vec)
             output = u_val
@@ -490,7 +428,7 @@ class IOSystemFactory(object):
             return output
         #
         return control.NonlinearIOSystem(
-            None, outfcn, outputs=[OUT], inputs=[IN], name=name)
+            None, outfcn, outputs=[output_name], inputs=[input_name], name=name)
 
     def makeMultiplier(self, name, factor):
         """
